from .akiva import AKIVA as AKIVA
from .allinitu import ALLINITU as ALLINITU
from .arglina import ARGLINA as ARGLINA
from .arglinb import ARGLINB as ARGLINB
from .arglinc import ARGLINC as ARGLINC
from .argtrigls import ARGTRIGLS as ARGTRIGLS
from .arwhead import ARWHEAD as ARWHEAD

# TODO: BA_L family needs human review
# from .ba_l1ls import BA_L1LS as BA_L1LS  # TODO: BA_L family needs human review
# from .ba_l1spls import BA_L1SPLS as BA_L1SPLS  # TODO: Human review - Hessian fails
from .bard import BARD as BARD
from .bdqrtic import BDQRTIC as BDQRTIC
from .beale import BEALE as BEALE
from .bennett5ls import BENNETT5LS as BENNETT5LS
from .biggs6 import BIGGS6 as BIGGS6
from .box import BOX as BOX
from .box3 import BOX3 as BOX3
from .boxbodls import BOXBODLS as BOXBODLS

# TODO: BOXPOWER needs human review - minor gradient discrepancy at last element
# from .boxpower import BOXPOWER as BOXPOWER
# from .bratu1d import BRATU1D as BRATU1D  # TODO: Human review needed - see file
# TODO: BRKMCC needs human review - significant discrepancies
# from .brkmcc import BRKMCC as BRKMCC
# TODO: BROWNAL needs human review - small Hessian discrepancies
# from .brownal import BROWNAL as BROWNAL
from .brownbs import BROWNBS as BROWNBS
from .brownden import BROWNDEN as BROWNDEN
from .broydn3dls import BROYDN3DLS as BROYDN3DLS
from .broydn7d import BROYDN7D as BROYDN7D

# TODO: BROYDNBDLS and BRYBND require human review - gradient tests fail
# from .broydnbdls import BROYDNBDLS as BROYDNBDLS
# from .brybnd import BRYBND as BRYBND
# TODO: CERI problems need human review - numerical instability in erfc/exp
# from .ceri import (
#     CERI651ALS as CERI651ALS,
#     CERI651BLS as CERI651BLS,
#     CERI651CLS as CERI651CLS,
#     CERI651DLS as CERI651DLS,
#     CERI651ELS as CERI651ELS,
# )
from .chainwoo import CHAINWOO as CHAINWOO
from .chnrosnb import CHNROSNB as CHNROSNB
from .chnrsnbm import CHNRSNBM as CHNRSNBM

# TODO: CHWIRUT1 needs external data file with 214 data points
# from .chwirut1 import CHWIRUT1 as CHWIRUT1
from .chwirut1ls import CHWIRUT1LS as CHWIRUT1LS

# TODO: CHWIRUT2 needs implementation with 54 data points
# from .chwirut2 import CHWIRUT2 as CHWIRUT2
from .chwirut2ls import CHWIRUT2LS as CHWIRUT2LS
from .cliff import CLIFF as CLIFF
from .clusterls import CLUSTERLS as CLUSTERLS
from .coating import COATING as COATING
from .coolhansls import COOLHANSLS as COOLHANSLS
from .cosine import COSINE as COSINE
from .cragglvy import CRAGGLVY as CRAGGLVY
from .cube import CUBE as CUBE
from .curly10 import CURLY10 as CURLY10
from .curly20 import CURLY20 as CURLY20
from .curly30 import CURLY30 as CURLY30
from .cyclic3ls import CYCLIC3LS as CYCLIC3LS
from .cycloocfls import CYCLOOCFLS as CYCLOOCFLS
from .daniwoodls import DANIWOODLS as DANIWOODLS
from .denschna import DENSCHNA as DENSCHNA
from .denschnb import DENSCHNB as DENSCHNB
from .denschnc import DENSCHNC as DENSCHNC
from .denschnd import DENSCHND as DENSCHND
from .denschne import DENSCHNE as DENSCHNE
from .denschnf import DENSCHNF as DENSCHNF
from .devgla1 import DEVGLA1 as DEVGLA1
from .devgla2 import DEVGLA2 as DEVGLA2

# TODO: Human review needed - Likely gradient/Hessian precision issues
# from .dmn15332ls import DMN15332LS as DMN15332LS
# from .dmn15333ls import DMN15333LS as DMN15333LS
# from .dmn37142ls import DMN37142LS as DMN37142LS
# from .dmn37143ls import DMN37143LS as DMN37143LS
# from .diamon3dls import DIAMON3DLS as DIAMON3DLS  # TODO: Human review needed
from .dixmaana1 import DIXMAANA1 as DIXMAANA1
from .dixmaanb import DIXMAANB as DIXMAANB
from .dixmaanc import DIXMAANC as DIXMAANC
from .dixmaand import DIXMAAND as DIXMAAND
from .dixmaane1 import DIXMAANE1 as DIXMAANE1
from .dixmaanf import DIXMAANF as DIXMAANF
from .dixmaang import DIXMAANG as DIXMAANG
from .dixmaanh import DIXMAANH as DIXMAANH
from .dixmaani1 import DIXMAANI1 as DIXMAANI1
from .dixmaanj import DIXMAANJ as DIXMAANJ
from .dixmaank import DIXMAANK as DIXMAANK
from .dixmaanl import DIXMAANL as DIXMAANL
from .dixmaanm1 import DIXMAANM1 as DIXMAANM1
from .dixmaann import DIXMAANN as DIXMAANN
from .dixmaano import DIXMAANO as DIXMAANO
from .dixmaanp import DIXMAANP as DIXMAANP
from .dixon3dq import DIXON3DQ as DIXON3DQ
from .djtl import DJTL as DJTL
from .dmn15102ls import DMN15102LS as DMN15102LS
from .dmn15103ls import DMN15103LS as DMN15103LS
from .dqdrtic import DQDRTIC as DQDRTIC
from .dqrtic import DQRTIC as DQRTIC
from .drcav1lq import DRCAV1LQ as DRCAV1LQ
from .drcav2lq import DRCAV2LQ as DRCAV2LQ
from .drcav3lq import DRCAV3LQ as DRCAV3LQ

# TODO: ECKERLE4LS needs human review - significant discrepancies
# from .eckerle4ls import ECKERLE4LS as ECKERLE4LS
from .edensch import EDENSCH as EDENSCH
from .eg1 import EG1 as EG1
from .eg2 import EG2 as EG2
from .eggcrate import EGGCRATE as EGGCRATE
from .eigenals import EIGENALS as EIGENALS
from .eigenbls import EIGENBLS as EIGENBLS
from .eigencls import EIGENCLS as EIGENCLS
from .elatvidu import ELATVIDU as ELATVIDU
from .engval1 import ENGVAL1 as ENGVAL1
from .engval2 import ENGVAL2 as ENGVAL2

# TODO: ENSOLS needs human review - significant discrepancies
# from .ensols import ENSOLS as ENSOLS
from .errinros import ERRINROS as ERRINROS

# TODO: ERRINRSM needs human review - significant discrepancies
# from .errinros import ERRINRSM as ERRINRSM
from .exp_scipy import EXP2 as EXP2
from .expfit import EXPFIT as EXPFIT

# TODO: EXTROSNB needs human review - objective/gradient discrepancies
# from .extrosnb import EXTROSNB as EXTROSNB
from .fbrain3ls import FBRAIN3LS as FBRAIN3LS
from .fletbv3m import FLETBV3M as FLETBV3M
from .fletcbv2 import FLETCBV2 as FLETCBV2
from .fletcbv3 import FLETCBV3 as FLETCBV3
from .fletchcr import FLETCHCR as FLETCHCR

# TODO: Human review - objective/gradient discrepancies
# from .fletchbv import FLETCHBV as FLETCHBV
# TODO: FMINSURF and FMINSRF2 have bugs - starting value/gradient discrepancies
# from .fminsurf import FMINSRF2 as FMINSRF2, FMINSURF as FMINSURF
# TODO: FREURONE needs human review - miscategorized (should be constrained)
# from .freuroth import FREURONE as FREURONE
from .freuroth import FREUROTH as FREUROTH

# TODO: GAUSS family needs human review - issues reported by user
# from .gauss import GAUSS1LS as GAUSS1LS, GAUSS2LS as GAUSS2LS, GAUSS3LS as GAUSS3LS
from .gaussian import GAUSSIAN as GAUSSIAN

# TODO: GBRAINLS needs human review - complex data dependencies
# from .gbrainls import GBRAINLS as GBRAINLS
from .genhumps import GENHUMPS as GENHUMPS
from .genrose import GENROSE as GENROSE
from .growthls import GROWTHLS as GROWTHLS

# TODO: GULF needs human review - issues reported by user
# from .gulf import GULF as GULF
from .hahn1ls import HAHN1LS as HAHN1LS
from .hairy import HAIRY as HAIRY
from .hatfldd import HATFLDD as HATFLDD
from .hatflde import HATFLDE as HATFLDE
from .hatfldfl import HATFLDFL as HATFLDFL
from .hatfldfls import HATFLDFLS as HATFLDFLS

# from .hatfldgls import HATFLDGLS as HATFLDGLS  # TODO: gradient/Hessian issues
# TODO: HEART problems need human review - significant discrepancies
# from .heart import HEART6LS as HEART6LS, HEART8LS as HEART8LS
from .helix import HELIX as HELIX

# TODO: HIELOW needs human review - significant discrepancies
# from .hielow import HIELOW as HIELOW
from .hilberta import HILBERTA as HILBERTA
from .hilbertb import HILBERTB as HILBERTB

# TODO: HIMMELBB needs human review - Hessian H[0,0] differs from pycutest
# from .himmelbb import HIMMELBB as HIMMELBB
from .himmelbcls import HIMMELBCLS as HIMMELBCLS

# TODO: HIMMELBF needs human review - Hessian cross-derivative H[2,3] differs
# from .himmelbf import HIMMELBF as HIMMELBF
from .himmelbg import HIMMELBG as HIMMELBG
from .himmelbh import HIMMELBH as HIMMELBH
from .humps import HUMPS as HUMPS
from .indef import INDEF as INDEF
from .indefm import INDEFM as INDEFM
from .inteqnels import INTEQNELS as INTEQNELS
from .jensmp import JENSMP as JENSMP
from .judge import JUDGE as JUDGE
from .kirby import KIRBY2LS as KIRBY2LS
from .kowosb import KOWOSB as KOWOSB

# TODO: KSSLS needs human review - significant objective/gradient discrepancies
# from .kssls import KSSLS as KSSLS
from .lanczos1ls import LANCZOS1LS as LANCZOS1LS
from .lanczos2ls import LANCZOS2LS as LANCZOS2LS
from .liarwhd import LIARWHD as LIARWHD
from .loghairy import LOGHAIRY as LOGHAIRY
from .lsc1ls import LSC1LS as LSC1LS
from .lsc2ls import LSC2LS as LSC2LS
from .luksan11ls import LUKSAN11LS as LUKSAN11LS
from .luksan12ls import LUKSAN12LS as LUKSAN12LS
from .luksan13ls import LUKSAN13LS as LUKSAN13LS
from .luksan14ls import LUKSAN14LS as LUKSAN14LS
from .luksan15ls import LUKSAN15LS as LUKSAN15LS
from .luksan16ls import LUKSAN16LS as LUKSAN16LS
from .luksan17ls import LUKSAN17LS as LUKSAN17LS
from .luksan21ls import LUKSAN21LS as LUKSAN21LS

# from .luksan22ls import LUKSAN22LS as LUKSAN22LS  # TODO: Human review - gradients
# TODO: MANCINO needs human review - significant discrepancies in all values
# from .mancino import MANCINO as MANCINO
from .maratosb import MARATOSB as MARATOSB
from .mexhat import MEXHAT as MEXHAT
from .mgh09ls import MGH09LS as MGH09LS
from .mgh10ls import MGH10LS as MGH10LS
from .mgh10sls import MGH10SLS as MGH10SLS
from .mgh17ls import MGH17LS as MGH17LS
from .mgh17sls import MGH17SLS as MGH17SLS

# TODO: Human review needed - Minor gradient precision differences
# from .morebv import MOREBV as MOREBV
# TODO: Human review - SCALE interpretation issue
# from .modbeale import MODBEALE as MODBEALE
# from .nondia import NONDIA as NONDIA  # TODO: Human review - SCALE factor issue
from .noncvxu2 import NONCVXU2 as NONCVXU2
from .noncvxun import NONCVXUN as NONCVXUN
from .nondquar import NONDQUAR as NONDQUAR
from .nonmsqrt import NONMSQRT as NONMSQRT
from .osbornea import OSBORNEA as OSBORNEA

# TODO: Human review - objective discrepancy
# from .osborneb import OSBORNEB as OSBORNEB
from .palmer1c import PALMER1C as PALMER1C
from .palmer1d import PALMER1D as PALMER1D
from .palmer2c import PALMER2C as PALMER2C
from .palmer3c import PALMER3C as PALMER3C
from .palmer4c import PALMER4C as PALMER4C
from .palmer5c import PALMER5C as PALMER5C
from .palmer5d import PALMER5D as PALMER5D
from .palmer6c import PALMER6C as PALMER6C
from .palmer7c import PALMER7C as PALMER7C
from .palmer8c import PALMER8C as PALMER8C

# TODO: Human review - minor numerical precision issues
# from .penalty1 import PENALTY1 as PENALTY1
# TODO: Human review - SCALE factor issue
# from .penalty2 import PENALTY2 as PENALTY2
from .penalty3 import PENALTY3 as PENALTY3
from .power import POWER as POWER
from .powersum import POWERSUM as POWERSUM
from .price3 import PRICE3 as PRICE3
from .price4 import PRICE4 as PRICE4

# TODO: Human review - objective off by factor of 4.15
# from .powellsg import POWELLSG as POWELLSG
from .quartc import QUARTC as QUARTC
from .rosenbr import ROSENBR as ROSENBR
from .roszman1ls import ROSZMAN1LS as ROSZMAN1LS
from .s308 import S308 as S308

# from .sparsqur import SPARSQUR as SPARSQUR  # TODO: Human review - Hessian timeout
# from .sscosine import SSCOSINE as SSCOSINE  # TODO: Human review needed
from .sbrybnd import SBRYBND as SBRYBND

# TODO: SCHMVETT needs human review - Hessian NaN issue at zero vector
# from .schmvett import SCHMVETT as SCHMVETT
# from .scosine import SCOSINE as SCOSINE  # TODO: Human review needed
from .scurly10 import SCURLY10 as SCURLY10
from .scurly20 import SCURLY20 as SCURLY20
from .scurly30 import SCURLY30 as SCURLY30

# from .sineval import SINEVAL as SINEVAL  # TODO: Human review - Complex SCALE
# from .sineali import SINEALI as SINEALI  # TODO: Human review - Should be in bounded
from .sisser import SISSER as SISSER
from .snail import SNAIL as SNAIL
from .sparsine import SPARSINE as SPARSINE

# from .sensors import SENSORS as SENSORS  # TODO: Human review - pycutest issues
# from .sinquad import SINQUAD as SINQUAD  # TODO: Human review - Complex SIF
from .spin2ls import SPIN2LS as SPIN2LS
from .srosenbr import SROSENBR as SROSENBR

# TODO: Human review needed - gradient/Hessian issues with auxiliary variables
# from .spinls import SPINLS as SPINLS
# TODO: Human review needed - complex matrix multiplication formulation
# from .spmsrtls import SPMSRTLS as SPMSRTLS
from .tenfolds import TENFOLDTRLS as TENFOLDTRLS

# TODO: TOINTGOR needs human review - runtime test fails (~8x slower than threshold)
# from .tointgor import TOINTGOR as TOINTGOR
from .tointgss import TOINTGSS as TOINTGSS

# TODO: TOINTPSP needs human review - gradient test fails with small differences
# from .tointpsp import TOINTPSP as TOINTPSP
# TODO: TQUARTIC needs human review - objective calculation incorrect
# from .tquartic import TQUARTIC as TQUARTIC
from .trigon1 import TRIGON1 as TRIGON1

# TODO: TRIGON2 needs human review - Hessian test fails
# from .trigon2 import TRIGON2 as TRIGON2
from .vandanmsls import VANDANMSLS as VANDANMSLS
from .vardim import VARDIM as VARDIM

# TODO: Human review - matrix computation discrepancy
# from .vareigvl import VAREIGVL as VAREIGVL
from .vesuvials import VESUVIALS as VESUVIALS
from .vesuviols import VESUVIOLS as VESUVIOLS
from .vesuviouls import VESUVIOULS as VESUVIOULS
from .vibrbeam import VIBRBEAM as VIBRBEAM

# TODO: Human review needed - Hessian computation issues
# from .watson import WATSON as WATSON
from .waysea1 import WAYSEA1 as WAYSEA1
from .waysea2 import WAYSEA2 as WAYSEA2
from .woods import WOODS as WOODS
from .yatp1cls import YATP1CLS as YATP1CLS
from .yatp1ls import YATP1LS as YATP1LS
from .yatp2cls import YATP2CLS as YATP2CLS

# TODO: Human review needed - Hessian test failures
# Attempted fixes:
# 1. Corrected z_i vs z_j bug (now uses z_j as per SIF)
# 2. Verified residual ordering matches YATP1LS
# 3. Investigated JAX autodiff vs pycutest analytical derivatives
# Issue: Hessian elements 6, 17, 18 differ significantly (~13.45)
# from .yatp2ls import YATP2LS as YATP2LS
from .zangwil2 import ZANGWIL2 as ZANGWIL2


unconstrained_minimisation_problems = (
    AKIVA(),
    ALLINITU(),
    ARGLINA(),
    ARGLINB(),
    ARGLINC(),
    ARGTRIGLS(),
    ARWHEAD(),
    # BA_L1LS(),  # TODO: BA_L family needs human review
    # BA_L1SPLS(),  # TODO: Human review - Hessian test fails
    BARD(),
    BDQRTIC(),
    BEALE(),
    BENNETT5LS(),
    BIGGS6(),
    BOX(),
    BOX3(),
    BOXBODLS(),
    # BOXPOWER(),  # TODO: Human review - minor gradient discrepancy at last element
    # BRATU1D(),  # TODO: Human review needed - see file
    # BRKMCC(),  # TODO: Human review - significant discrepancies
    # BROWNAL(),  # TODO: Human review - small Hessian discrepancies
    BROWNBS(),
    BROWNDEN(),
    BROYDN3DLS(),
    BROYDN7D(),
    # BROYDNBDLS(),  # TODO: Gradient test fails - needs human review
    # BRYBND(),  # TODO: Gradient test fails - needs human review
    # CERI651ALS(),  # TODO: Human review - numerical instability in erfc/exp
    # CERI651BLS(),  # TODO: Human review - numerical instability in erfc/exp
    # CERI651CLS(),  # TODO: Human review - numerical instability in erfc/exp
    # CERI651DLS(),  # TODO: Human review - numerical instability in erfc/exp
    # CERI651ELS(),  # TODO: Human review - numerical instability in erfc/exp
    CHAINWOO(),
    CHNROSNB(),
    CHNRSNBM(),
    CHWIRUT1LS(),
    CHWIRUT2LS(),
    CLIFF(),
    CLUSTERLS(),
    COATING(),
    COOLHANSLS(),
    COSINE(),
    CRAGGLVY(),
    CUBE(),
    CURLY10(),
    CURLY20(),
    CURLY30(),
    SCURLY10(),
    SCURLY20(),
    SCURLY30(),
    CYCLIC3LS(),
    CYCLOOCFLS(),
    DANIWOODLS(),
    DENSCHNA(),
    DENSCHNB(),
    DENSCHNC(),
    DENSCHND(),
    DENSCHNE(),
    DENSCHNF(),
    DEVGLA1(),
    # DIAMON3DLS(),  # TODO: Human review needed - see file
    DEVGLA2(),
    DMN15102LS(),
    DMN15103LS(),
    # DMN15332LS(),  # TODO: Human review needed - Gradient/Hessian precision issues
    # DMN15333LS(),  # TODO: Human review needed - Gradient/Hessian precision issues
    # DMN37142LS(),  # TODO: Human review needed - Gradient/Hessian precision issues
    # DMN37143LS(),  # TODO: Human review needed - Gradient/Hessian precision issues
    DIXMAANA1(),
    DIXMAANB(),
    DIXMAANC(),
    DIXMAAND(),
    DIXMAANE1(),
    DIXMAANF(),
    DIXMAANG(),
    DIXMAANH(),
    DIXMAANI1(),
    DIXMAANJ(),
    DIXMAANK(),
    DIXMAANL(),
    DIXMAANM1(),
    DIXMAANN(),
    DIXMAANO(),
    DIXMAANP(),
    DIXON3DQ(),
    DRCAV1LQ(),
    DRCAV2LQ(),
    DRCAV3LQ(),
    DJTL(),
    DQDRTIC(),
    DQRTIC(),
    # ECKERLE4LS(),  # TODO: Human review - significant discrepancies
    EDENSCH(),
    EG2(),
    EGGCRATE(),
    EIGENALS(),
    EIGENBLS(),
    EIGENCLS(),
    ELATVIDU(),
    ENGVAL1(),
    ENGVAL2(),
    # ENSOLS(),  # TODO: Human review - significant discrepancies
    ERRINROS(),
    # ERRINRSM(),  # TODO: Human review - significant discrepancies
    EXP2(),
    EXPFIT(),
    # EXTROSNB(),  # TODO: Human review - objective/gradient discrepancies
    FBRAIN3LS(),
    # FLETCH family problems
    # FLETCHBV(),  # TODO: Human review - objective/gradient discrepancies
    FLETBV3M(),
    FLETCBV2(),
    FLETCHCR(),
    # Not varying the scale term in the FLETCBV3 problem
    FLETCBV3(),
    #    FMINSURF(),  # TODO: has a bug
    #    FMINSRF2(),  # TODO: has a bug
    FREUROTH(),
    # FREURONE(),  # TODO: Human review - miscategorized (should be constrained)
    # GAUSS1LS(),  # TODO: Human review - issues reported by user
    # GAUSS2LS(),  # TODO: Human review - issues reported by user
    # GAUSS3LS(),  # TODO: Human review - issues reported by user
    GAUSSIAN(),
    # GBRAINLS(),  # TODO: Human review - complex data dependencies
    GENHUMPS(),
    GENROSE(),
    GROWTHLS(),
    # GULF(),  # TODO: Human review - issues reported by user
    HAHN1LS(),
    # HAHN1LS(y0_id=1),  # Non-default starting point - we only test pycutest defaults
    HAIRY(),
    HATFLDD(),
    HATFLDE(),
    HATFLDFL(),
    HATFLDFLS(),
    # HATFLDGLS(),  # TODO: gradient/Hessian issues
    # HEART6LS(),  # TODO: Human review - significant discrepancies
    # HEART8LS(),  # TODO: Human review - significant discrepancies
    HELIX(),
    # HIELOW(),  # TODO: Human review - significant discrepancies
    HILBERTA(),
    HILBERTB(),
    # HIMMELBB(),  # TODO: needs human review - Hessian issues
    HIMMELBCLS(),
    # HIMMELBF(),  # TODO: needs human review - Hessian issues
    HIMMELBG(),
    HIMMELBH(),
    HUMPS(),
    INDEF(),
    INDEFM(),
    INTEQNELS(),
    JENSMP(),
    JUDGE(),
    KIRBY2LS(),
    KOWOSB(),
    # KSSLS(),  # TODO: Human review - significant objective/gradient discrepancies
    LANCZOS1LS(),
    LANCZOS2LS(),
    LIARWHD(),
    LOGHAIRY(),
    # LR(),
    LSC1LS(),
    LSC2LS(),
    LUKSAN11LS(),
    LUKSAN12LS(),
    LUKSAN13LS(),
    LUKSAN14LS(),
    LUKSAN15LS(),
    LUKSAN16LS(),
    LUKSAN17LS(),
    LUKSAN21LS(),
    # LUKSAN22LS(),  # TODO: Human review needed - gradient issues
    # MANCINO(),  # TODO: Human review - significant discrepancies in all values
    MARATOSB(),
    MEXHAT(),
    MGH09LS(),
    MGH10LS(),
    MGH10SLS(),
    MGH17LS(),
    MGH17SLS(),
    # MOREBV(),  # TODO: Human review - minor gradient precision differences
    # MODBEALE(),  # TODO: Human review - SCALE interpretation issue
    # NONDIA(),  # TODO: Human review - SCALE factor issue
    NONCVXU2(),
    NONCVXUN(),
    NONDQUAR(),
    NONMSQRT(),
    OSBORNEA(),
    # OSBORNEB(),  # TODO: Human review - objective discrepancy
    PALMER1C(),
    PALMER1D(),
    PALMER2C(),
    PALMER3C(),
    PALMER4C(),
    PALMER5C(),
    PALMER5D(),
    PALMER6C(),
    PALMER7C(),
    PALMER8C(),
    # PENALTY1(),  # TODO: Human review - minor numerical precision issues
    # PENALTY2(),  # TODO: Human review - SCALE factor issue
    PENALTY3(),
    POWER(),
    POWERSUM(),
    # POWELLSG(),  # TODO: Human review - objective off by factor of 4.15
    PRICE3(),
    PRICE4(),
    QUARTC(),
    ROSENBR(),
    ROSZMAN1LS(),
    S308(),
    # SCOSINE(),  # TODO: Human review needed
    SCURLY10(),
    SCURLY20(),
    SCURLY30(),
    # SINEVAL(),  # TODO: Human review - Complex SCALE parameter interpretation
    # SINEALI(),  # TODO: Human review - Should be in bounded_minimisation module
    SISSER(),
    SNAIL(),
    SPARSINE(),
    # SPARSQUR(),  # TODO: Human review - Hessian tests timeout
    SBRYBND(),
    # SCHMVETT(),  # TODO: Human review - Hessian NaN issue at zero vector
    # SENSORS(),  # TODO: Human review - pycutest issues
    # SINQUAD(),  # TODO: Human review - Complex SIF
    SROSENBR(),
    # SSCOSINE(),  # TODO: Human review needed
    # SPINLS(),  # TODO: Human review - gradient/Hessian issues
    SPIN2LS(),
    # SPMSRTLS(),  # TODO: Human review - complex matrix multiplication
    TENFOLDTRLS(),
    # WATSON(),  # TODO: Human review - Hessian computation issues
    WAYSEA1(),
    WAYSEA2(),
    WOODS(),
    YATP1CLS(),
    YATP1LS(),
    YATP2CLS(),
    # YATP2LS(),  # TODO: Human review - Hessian test failures
    ZANGWIL2(),
    TRIGON1(),
    # TRIGON2(),  # TODO: Human review - Hessian test fails
    # TOINTGOR(),  # TODO: Human review - runtime test fails
    TOINTGSS(),
    # TOINTPSP(),  # TODO: Human review - gradient test fails
    # TQUARTIC(),  # TODO: Human review - objective calculation incorrect
    VANDANMSLS(),
    VARDIM(),
    # VAREIGVL(),  # TODO: Human review - matrix computation discrepancy
    VESUVIALS(),
    VIBRBEAM(),
    VESUVIOLS(),
    VESUVIOULS(),
)
